@@ -26,6 +26,51 @@ impl WebPDecoderConfig {
     }
 }
 
+pub unsafe fn WebPInitConfig(config: *mut WebPConfig) -> bool {
+    WebPConfigInitInternal(config, WebPPreset::WEBP_PRESET_DEFAULT, 75.0, WEBP_DECODER_ABI_VERSION) != 0
+}
+
+impl WebPConfig {
+    pub fn new() -> Result<Self, ()> {
+        unsafe {
+            let mut out = std::mem::MaybeUninit::uninit();
+            if WebPInitConfig(out.as_mut_ptr()) {
+                Ok(out.assume_init())
+            } else {
+                Err(())
+            }
+        }
+    }
+
+    pub fn new_with_preset(preset: WebPPreset, quality: f32) -> Result<Self, ()> {
+        unsafe {
+            let mut out = std::mem::MaybeUninit::uninit();
+            if WebPConfigInitInternal(out.as_mut_ptr(), preset, quality, WEBP_DECODER_ABI_VERSION) != 0 {
+                Ok(out.assume_init())
+            } else {
+                Err(())
+            }
+        }
+    }
+}
+
+pub unsafe fn WebPPictureInit(config: *mut WebPPicture) -> bool {
+    WebPPictureInitInternal(config, WEBP_DECODER_ABI_VERSION) != 0
+}
+
+impl WebPPicture {
+    pub fn new() -> Result<Self, ()> {
+        unsafe {
+            let mut out = std::mem::MaybeUninit::uninit();
+            if WebPPictureInit(out.as_mut_ptr()) {
+                Ok(out.assume_init())
+            } else {
+                Err(())
+            }
+        }
+    }
+}
+
 pub unsafe fn WebPGetFeatures(
     arg1: *const u8,
     arg2: usize,
@@ -113,6 +158,45 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_webp_encode_advanced() {
+        use std::ffi::c_void;
+
+        let mut buf = Vec::new();
+        let len = File::open("./tests/test1_1000x1000.bif").unwrap().read_to_end(&mut buf).unwrap();
+        assert_eq!(4000000, len);
+
+        unsafe {
+            let config = WebPConfig::new().unwrap();
+            let mut picture = WebPPicture::new().unwrap();
+            picture.use_argb = 1;
+            picture.argb = buf.as_ptr() as *mut u32;
+            picture.width = 1000;
+            picture.height = 1000;
+            picture.argb_stride = 1000;
+
+            let mut out = vec![];
+
+            unsafe extern "C" fn writer_function(
+                data: *const u8,
+                data_size: usize,
+                picture: *const WebPPicture,
+            ) -> ::std::os::raw::c_int {
+                let mut out: &mut Vec<u8> = std::mem::transmute((*picture).custom_ptr);
+                out.extend_from_slice(std::slice::from_raw_parts(data, data_size));
+                0
+            }
+
+            picture.writer = Some(writer_function);
+            picture.custom_ptr = &mut out as *mut _ as *mut c_void;
+
+            assert_eq!(WebPEncode(&config, &mut picture), 0);
+
+            assert_eq!(b"RIFF", &out[0..4]);
+            assert_eq!(b"WEBP", &out[8..12]);
+        }
+    }
+
     #[test]
     fn test_webp_decode() {
         let mut buf = Vec::new();
